"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const ursa = require("ursa");
const shared_1 = require("./shared");
function autoGenerateKeys(keysPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const privateKeyPath = path.join(keysPath, "auth.key");
        const publicKeyPath = path.join(keysPath, "auth.pub");
        const publicKeyFileExists = yield fs.pathExists(publicKeyPath);
        const privateKeyFileExists = yield fs.pathExists(privateKeyPath);
        if (publicKeyFileExists && privateKeyFileExists) {
            return;
        }
        if (publicKeyFileExists && !privateKeyFileExists) {
            throw new Error(`The private key file at ${privateKeyPath} doesn't exist but the public key is available. Please supply both.`);
        }
        if (publicKeyFileExists && !privateKeyFileExists) {
            throw new Error(`The public key file at ${publicKeyPath} doesn't exist but the private key is available. Please supply both.`);
        }
        yield fs.mkdirp(keysPath);
        yield generatePrivateKey(privateKeyPath);
        yield exportPublicRSAKey(privateKeyPath, publicKeyPath);
        const adminTokenUserJsonLocation = path.resolve(keysPath, 'admin.json');
        if (yield fs.pathExists(adminTokenUserJsonLocation)) {
            return;
        }
        const privateKey = yield fs.readFile(privateKeyPath).then(b => b.toString());
        const adminToken = shared_1.generateAdminToken(privateKey);
        yield fs.writeFile(adminTokenUserJsonLocation, JSON.stringify({
            "ADMIN_TOKEN": adminToken
        }));
    });
}
exports.autoGenerateKeys = autoGenerateKeys;
function generatePrivateKey(privateKeyPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const keyData = ursa.generatePrivateKey(2048);
        yield fs.writeFile(privateKeyPath, keyData.toPrivatePem());
    });
}
function exportPublicRSAKey(privateKeyPath, publicKeyPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const privateKeyData = yield fs.readFile(privateKeyPath);
        const privateKey = ursa.createPrivateKey(privateKeyData);
        const alreadyExists = yield fs.pathExists(publicKeyPath);
        if (alreadyExists) {
            return;
        }
        yield fs.writeFile(publicKeyPath, privateKey.toPublicPem());
    });
}
//# sourceMappingURL=autokeygen.js.map