"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const httpProxy = require("http-proxy");
const decorators_1 = require("../decorators");
const logger_1 = require("../logger");
const realm_sync_server_1 = require("realm-sync-server");
const tokenhelpers_1 = require("../shared/tokenhelpers");
let SyncService = class SyncService {
    constructor(config) {
        this.logger = new logger_1.Mute();
        this.config = config;
    }
    setLogger(l) {
        this.logger = l;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            yield fs.mkdirp(this.config.dataPath);
            this.syncServer = new realm_sync_server_1.RealmSyncServer({
                dataPath: this.config.dataPath,
                publicKeyPath: this.config.publicKeyPath,
                listenAddress: this.config.listenAddress,
                listenPort: this.config.listenPort,
                logCallback: (levelNo, message) => {
                    this.logger.log(logger_1.SyncLogLevel[levelNo], message);
                }
            });
            yield this.syncServer.start();
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.syncServer) {
                yield this.syncServer.stop();
            }
        });
    }
    websocketHandler(req, socket, head) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = req.headers['x-realm-access-token'];
            const parsedToken = tokenhelpers_1.parseTokenFromBase64RSA_SHA256(token);
            if (!parsedToken) {
                socket.write('HTTP/1.1 400 Bad Request\r\nUpgrade Failed\r\n\r\n');
                socket.end();
                return;
            }
            const address = this.address();
            const target = `http://${address.address}:${address.port}`;
            const proxy = httpProxy.createProxyServer({ ws: true, target });
            proxy.on('error', (err) => {
                this.logger.error(`sync proxy error ${err}`);
            });
            proxy.on('close', () => {
                this.logger.error("sync proxy closed");
            });
            proxy.ws(req, socket, head);
        });
    }
    address() {
        if (this.syncServer) {
            return this.syncServer.address();
        }
    }
};
__decorate([
    decorators_1.Unmute(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [logger_1.Logger]),
    __metadata("design:returntype", void 0)
], SyncService.prototype, "setLogger", null);
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], SyncService.prototype, "start", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], SyncService.prototype, "stop", null);
__decorate([
    decorators_1.Upgrade('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SyncService.prototype, "websocketHandler", null);
SyncService = __decorate([
    decorators_1.ServiceName('sync'),
    decorators_1.BaseRoute('/realm-object-server'),
    __metadata("design:paramtypes", [Object])
], SyncService);
exports.SyncService = SyncService;
//# sourceMappingURL=SyncService.js.map