/// <reference types="express" />
import * as Realm from 'realm';
import { Logger } from '../../logger';
import { Request } from 'express';
export interface RealmAccessTokenResponse {
    access_token: {
        token: string;
        token_data: any;
    };
}
export interface RealmManagementServiceConfig {
    privateKeyPath: string;
    tags?: string[];
}
export declare class RealmManagementService {
    logger: Logger;
    privateKey: string;
    realm: Realm;
    tags: string[];
    config: RealmManagementServiceConfig;
    constructor(config: RealmManagementServiceConfig);
    private accessLevelForPath(path, userId);
    setLogger(l: Logger): void;
    accessToken(incomingRequest: Request): Promise<RealmAccessTokenResponse>;
    find(incomingRequest: Request): Promise<{}[]>;
    private validatedRealmFile(path);
    private realmFileJSON(realmFile, withPermissions);
    get(incomingRequest: Request): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions?: {
            key: string;
            value: string;
            type: "user" | "all" | "metadata_exact" | "metadata_regex";
            accessLevel: "none" | "read" | "write" | "admin";
        }[];
    }>;
    remove(incomingRequest: Request): Promise<void>;
    getPermissions(incomingRequest: Request): Promise<{
        key: string;
        value: string;
        type: "user" | "all" | "metadata_exact" | "metadata_regex";
        accessLevel: "none" | "read" | "write" | "admin";
    }[]>;
    createPermissions(incomingRequest: Request): Promise<{}>;
    start(): Promise<void>;
}
